@@ -142,12 +142,12 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
     return *this;
   }
   /// Polymorphic cloning.
-  virtual PCRE2Matcher *clone()
+  virtual PCRE2Matcher *clone() REFLEX_OVERRIDE
   {
     return new PCRE2Matcher(*this);
   }
   /// Reset this matcher's state to the initial state and when assigned new input.
-  virtual void reset(const char *opt = NULL)
+  virtual void reset(const char *opt = NULL) REFLEX_OVERRIDE
   {
     DBGLOG("PCRE2Matcher::reset()");
     flg_ = 0;
@@ -191,6 +191,7 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
   /// Set the pattern regex string to use with this matcher (the given pattern is shared and must be persistent).
   virtual PatternMatcher& pattern(const Pattern *pattern) ///< pointer to a regex string
     /// @returns this matcher.
+    REFLEX_OVERRIDE
   {
     PatternMatcher::pattern(pattern);
     compile();
@@ -199,6 +200,7 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
   /// Set the pattern regex string to use with this matcher.
   virtual PatternMatcher& pattern(const char *pattern) ///< regex string
     /// @returns this matcher.
+    REFLEX_OVERRIDE
   {
     PatternMatcher::pattern(pattern);
     compile();
@@ -207,6 +209,7 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
   /// Set the pattern regex string to use with this matcher.
   virtual PatternMatcher& pattern(const std::string& pattern) ///< regex string
     /// @returns this matcher.
+    REFLEX_OVERRIDE
   {
     PatternMatcher::pattern(pattern);
     compile();
@@ -215,7 +218,7 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
   /// Returns a pair of pointer and length of the captured match for n > 0 capture index or <text(),size() for n == 0.
   virtual std::pair<const char*,size_t> operator[](size_t n) ///< nth capture index > 0 or 0
     /// @returns pair.
-    const
+    const REFLEX_OVERRIDE
   {
     if (n == 0)
       return std::pair<const char*,size_t>(txt_, len_);
@@ -230,6 +233,7 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
   /// Returns the group capture identifier containing the group capture index >0 and name (or NULL) of a named group capture, or (1,NULL) by default
   virtual std::pair<size_t,const char*> group_id()
     /// @returns a pair of size_t and string
+    REFLEX_OVERRIDE
   {
     grp_ = 1;
     if (dat_ == NULL || pcre2_get_ovector_count(dat_) <= 1)
@@ -242,6 +246,7 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
   /// Returns the next group capture identifier containing the group capture index >0 and name (or NULL) of a named group capture, or (0,NULL) when no more groups matched
   virtual std::pair<size_t,const char*> group_next_id()
     /// @returns a pair of size_t and string
+    REFLEX_OVERRIDE
   {
     if (dat_ == NULL)
       return std::pair<size_t,const char*>(0, static_cast<const char*>(NULL)); // cast to appease MSVC 2010
@@ -311,6 +316,7 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
   /// The match method Const::SCAN, Const::FIND, Const::SPLIT, or Const::MATCH, implemented with PCRE2.
   virtual size_t match(Method method) ///< match method Const::SCAN, Const::FIND, Const::SPLIT, or Const::MATCH
     /// @returns nonzero when input matched the pattern using method Const::SCAN, Const::FIND, Const::SPLIT, or Const::MATCH.
+    REFLEX_OVERRIDE
   {
     DBGLOG("BEGIN PCRE2Matcher::match(%d)", method);
     reset_text();
